@@ -10,7 +10,7 @@ from decimal import Decimal
 from math import isnan, isinf
 from uuid import UUID
 
-from .schema_util import validate_type_model_types_errors
+from .schema_util import validate_type_model_errors
 from .type_model import TYPE_MODEL
 
 
@@ -18,10 +18,10 @@ def get_referenced_types(types, type_name, referenced_types=None):
     """
     Get a type's referenced type model
 
-    :param dict types: The map of user type name to user type model
+    :param dict types: The `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
     :param str type_name: The type name
     :param dict referenced_types: An optional map of referenced user type name to user type
-    :returns: The map of referenced user type name to user type model
+    :returns: The referenced `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
     """
 
     return _get_referenced_types(types, {'user': type_name}, referenced_types)
@@ -115,7 +115,7 @@ def validate_type(types, type_name, value, member_fqn=None):
     Type-validate a value using the schema-markdown user type model. Container values are duplicated
     since some member types are transformed during validation.
 
-    :param dict types: The map of user type name to user type model
+    :param dict types: The `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
     :param str type_name: The type name
     :param object value: The value object to validate
     :param str member_fqn: The fully-qualified member name
@@ -253,11 +253,11 @@ def _validate_type(types, type_, value, member_fqn=None):
         array_value_nullable = array_attr is not None and 'nullable' in array_attr and array_attr['nullable']
         for ix_array_value, array_value in enumerate(value_new):
             member_fqn_value = f'{ix_array_value}' if member_fqn is None else f'{member_fqn}.{ix_array_value}'
-            if array_value is None or (array_value_nullable and array_value == 'null'):
+            if array_value_nullable and (array_value is None or array_value == 'null'):
                 array_value = None
             else:
                 array_value = _validate_type(types, array_type, array_value, member_fqn_value)
-            _validate_attr(array_type, array_attr, array_value, member_fqn_value)
+                _validate_attr(array_type, array_attr, array_value, member_fqn_value)
             value_copy.append(array_value)
 
         # Return the validated, transformed copy
@@ -285,18 +285,18 @@ def _validate_type(types, type_, value, member_fqn=None):
             member_fqn_key = dict_key if member_fqn is None else f'{member_fqn}.{dict_key}'
 
             # Validate the key
-            if dict_key is None or (dict_key_nullable and dict_key == 'null'):
+            if dict_key_nullable and (dict_key is None or dict_key == 'null'):
                 dict_key = None
             else:
                 dict_key = _validate_type(types, dict_key_type, dict_key, member_fqn)
-            _validate_attr(dict_key_type, dict_key_attr, dict_key, member_fqn)
+                _validate_attr(dict_key_type, dict_key_attr, dict_key, member_fqn)
 
             # Validate the value
-            if dict_value is None or (dict_value_nullable and dict_value == 'null'):
+            if dict_value_nullable and (dict_value is None or dict_value == 'null'):
                 dict_value = None
             else:
                 dict_value = _validate_type(types, dict_type, dict_value, member_fqn_key)
-            _validate_attr(dict_type, dict_attr, dict_value, member_fqn_key)
+                _validate_attr(dict_type, dict_attr, dict_value, member_fqn_key)
 
             # Copy the key/value
             value_copy[dict_key] = dict_value
@@ -319,11 +319,11 @@ def _validate_type(types, type_, value, member_fqn=None):
 
             # Validate the value
             value_nullable = typedef_attr is not None and 'nullable' in typedef_attr and typedef_attr['nullable']
-            if value is None or (value_nullable and value == 'null'):
+            if value_nullable and (value is None or value == 'null'):
                 value_new = None
             else:
                 value_new = _validate_type(types, typedef['type'], value, member_fqn)
-            _validate_attr(type_, typedef_attr, value_new, member_fqn)
+                _validate_attr(type_, typedef_attr, value_new, member_fqn)
 
         # enum?
         elif 'enum' in user_type:
@@ -355,6 +355,7 @@ def _validate_type(types, type_, value, member_fqn=None):
                 member_name = member['name']
                 member_fqn_member = member_name if member_fqn is None else f'{member_fqn}.{member_name}'
                 member_optional = member.get('optional', False)
+                member_nullable = 'attr' in member and member['attr'].get('nullable', False)
 
                 # Missing non-optional member?
                 if member_name not in value_new:
@@ -363,9 +364,11 @@ def _validate_type(types, type_, value, member_fqn=None):
                 else:
                     # Validate the member value
                     member_value = value_new[member_name]
-                    if member_value is not None:
+                    if member_nullable and (member_value is None or member_value == 'null'):
+                        member_value = None
+                    else:
                         member_value = _validate_type(types, member['type'], member_value, member_fqn_member)
-                    _validate_attr(member['type'], member.get('attr'), member_value, member_fqn_member)
+                        _validate_attr(member['type'], member.get('attr'), member_value, member_fqn_member)
 
                     # Copy the validated member
                     value_copy[member_name] = member_value
@@ -393,11 +396,7 @@ def _member_error(type_, value, member_fqn, attr=None):
 
 
 def _validate_attr(type_, attr, value, member_fqn):
-    if value is None:
-        value_nullable = attr is not None and 'nullable' in attr and attr['nullable']
-        if not value_nullable:
-            raise _member_error(type_, value, member_fqn)
-    elif attr is not None:
+    if attr is not None:
         if 'eq' in attr and not value == attr['eq']:
             raise _member_error(type_, value, member_fqn, f'== {attr["eq"]}')
         if 'lt' in attr and not value < attr['lt']:
@@ -424,9 +423,9 @@ def get_struct_members(types, struct):
     """
     Iterate the struct's members (inherited members first)
 
-    :param dict types: The map of user type name to user type model
-    :param dict struct: The struct model
-    :returns: The array of struct member models
+    :param dict types: The `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
+    :param dict struct: The `struct model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Struct'>`__
+    :returns: An iterator of `struct member models <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='StructMember'>`__
     """
 
     if 'bases' in struct:
@@ -443,9 +442,9 @@ def get_enum_values(types, enum):
     """
     Iterate the enum's values (inherited values first)
 
-    :param dict types: The map of user type name to user type model
-    :param dict enum: The enum model
-    :returns: The array of enum value models
+    :param dict types: The `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
+    :param dict enum: The `enum model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Enum'>`__
+    :returns: An iterator of `enum value models <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='EnumValue'>`__
     """
 
     if 'bases' in enum:
@@ -458,41 +457,21 @@ def get_enum_values(types, enum):
         yield from enum['values']
 
 
-def validate_type_model_types(types):
+def validate_type_model(types):
     """
-    Validate a user type model's types
+    Validate a user type model
 
-    :param dict types: The map of user type name to user type model
-    :returns: The validated, transformed type-model types dict
+    :param dict types: The `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
+    :returns: The validated `type model <https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types'>`__
     :raises ValidationError: A validation error occurred
     """
 
     # Validate with the type model
-    validated_types = validate_type(TYPE_MODEL['types'], 'Types', types)
+    validated_types = validate_type(TYPE_MODEL, 'Types', types)
 
     # Do additional type model validation
-    errors = validate_type_model_types_errors(validated_types)
+    errors = validate_type_model_errors(validated_types)
     if errors:
         raise ValidationError('\n'.join(message for _, _, message in sorted(errors)))
 
     return validated_types
-
-
-def validate_type_model(type_model):
-    """
-    Validate a user type model
-
-    :param dict type_model: The user type model
-    :returns: The validated, transformed type model
-    :raises ValidationError: A validation error occurred
-    """
-
-    # Validate with the type model
-    validated_type_model = validate_type(TYPE_MODEL['types'], 'TypeModel', type_model)
-
-    # Do additional type model validation
-    errors = validate_type_model_types_errors(validated_type_model['types'])
-    if errors:
-        raise ValidationError('\n'.join(message for _, _, message in sorted(errors)))
-
-    return validated_type_model
