@@ -5,21 +5,11 @@
 The schema-markdown type model definition
 """
 
-from .parser import SchemaMarkdownParser
-
-
-#: The Schema Markdown type model defined as Schema Markdown.
-TYPE_MODEL_SMD = '''\
-# A type model with a title
-struct TypeModel
-
-    # The type model's title
-    string title
-
-    # The type model
-    Types types
+from .parser import parse_schema_markdown
 
 
+#: The Schema Markdown type model
+TYPE_MODEL = parse_schema_markdown('''\
 # Map of user type name to user type model
 typedef UserType{len > 0} Types
 
@@ -252,11 +242,4 @@ struct ActionURL
 
     # The URL path. If not provided, uses the default URL path of "/<actionName>".
     optional string path
-'''
-
-
-#: The Schema Markdown type model.
-TYPE_MODEL = {
-    'title': 'The Schema Markdown Type Model',
-    'types': SchemaMarkdownParser(TYPE_MODEL_SMD).types
-}
+''')
